/**
 * This file includes functions for interacting with an ADC.
 *
 * Written for: TI ADS8698 8-channel ADC
 *
 * Connections:
 *    Function | ADC Pin     | AVR Port   | Pin # on DIP Atmega328P
 *   ----------+-------------+------------+-------------------------
 *    Data In  |  1 (SDI)    | PB3 (MOSI) | 17
 *    Reset    |  2 (RST)    | PC0        | 23
 *    Data Out | 36 (SDO)    | PB4 (MISO) | 18
 *    Clock    | 37 (SCLK)   | PB5 (SCK)  | 19
 *    Chip Sel.| 38 (CS)     | PC1        | 24
 */

#include <avr/io.h>
#include <util/delay.h>

#include "include/spi.h"
#include "include/ads8698.h"

/* Initialize the ADC. */
void ADS8698_init(void) {

    // Start SPI.
    SPI_init();

    // Set RST and CS as outputs.
    DDRC |= (1<<DDC0) | (1<<DDC1);

    // Pull RST and CS high.
    PORTC |= _BV(PORTC0);
    PORTC |= _BV(PORTC1);

    // Reset the ADC.
    ADS8698_reset();
}

/* Set the input range on the ADC. */
void ADS8698_set_range(ADS8698_range range) {
    uint8_t address;
    for (uint8_t channel = 0; channel < 8; channel++) {
        address = 0x05 + channel;
        ADS8698_write_register(address, range);
        ADS8698_read_register(address);
    }
}

/* Start the ADC's auto-scan mode. */
void ADS8698_auto_scan(void) {
    return; // TODO
}

/* Send a command to the ADC.
 *
 * See datasheet section 8.5.1.
 */
void ADS8698_send_command(ADS8698_command command) {
    ADS8698_CS_LOW;
    SPI_transmit(command);  // First byte
    SPI_transmit(0x00);  // Second byte (always zero).
    ADS8698_CS_HIGH;
}

/* Write to a register on the ADC.
 *
 * See datasheet section 8.5.2.
 */
void ADS8698_write_register(uint8_t address, uint8_t data) {
    ADS8698_CS_LOW;
    address |= 1UL << 7;  // 8th bit signifies "Read/Write Mode"; set 1.
    SPI_transmit(address);
    SPI_transmit(data);
    ADS8698_CS_HIGH;
}

/* Read from a register on the ADC.
 *
 * See datasheet section 8.5.2.
 */
uint8_t ADS8698_read_register(uint8_t address) {
    ADS8698_CS_LOW;
    address |= 0UL << 7;  // 8th bit signifies "Read/Write Mode"; set 0.
    SPI_transmit(address << 1);
    uint8_t data = SPI_receive();
    ADS8698_CS_HIGH;
    return data;
}


/* Reset the ADC.
 *
 * See datasheet section 8.4.1.1.6.
 */
void ADS8698_reset(void) {
    PORTC &= ~_BV(PORTC0);
    // Pull RST low for 70ns (instructions: 40-100ns).
    _delay_us(0.47);
    PORTC |= _BV(PORTC0);
}
